/*
    Dice component for Java
    Copyright (c) 1998 by Alessandro Scotti
*/
package org.ascotti.yawn;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.lang.*;

/*
*/
public class Dice extends Component
{
    private int fTag = 0;
    private int fValue = 1;
    private String fLabel = "";
    private ActionListener actionListener;
    private transient Random randomGenerator;

    public Dice()
    {
        this( "" );
    }

    public Dice( String l )
    {
        fLabel = l;
        setBackground( Color.red );
        setForeground( Color.black );
        setSize( 32, 32 );
        enableEvents(AWTEvent.MOUSE_EVENT_MASK);
        randomGenerator = new Random();
    }

    private void paintDots( Graphics g, int dots )
    {
        int radius = (getSize().width * 2) / 7;
        int offset = (getSize().width % radius) / 2 + 1;

        while( dots != 0 ) {
            int d = (dots & 0xF) - 1;
            int x = (d % 3)*radius + offset;
            int y = (d / 3)*radius + offset;

            g.fillOval( x, y, radius-1, radius-1 );
            dots >>>= 4;
        }
    }

    public void paint( Graphics g )
    {
        if( isVisible() ) {
            super.paint( g );
            g.setColor( getBackground() );
            g.fillRect( 0, 0, getSize().width, getSize().height );
            g.setColor( getForeground() );
            switch( fValue ) {
                case 1: paintDots( g, 0x5 ); break;
                case 2: paintDots( g, 0x19 ); break;
                case 3: paintDots( g, 0x159 ); break;
                case 4: paintDots( g, 0x1379 ); break;
                case 5: paintDots( g, 0x13579 ); break;
                case 6: paintDots( g, 0x134679 ); break;
            }
        }
    }

    public void processMouseEvent( MouseEvent e )
    {
        if( e.getID() == MouseEvent.MOUSE_PRESSED ) {
            if( actionListener != null )
                actionListener.actionPerformed( new ActionEvent( this, ActionEvent.ACTION_PERFORMED, "click" ) );
        }
        super.processMouseEvent( e );
    }

    public void addActionListener( ActionListener l )
    {
        actionListener = AWTEventMulticaster.add( actionListener, l );
        enableEvents( AWTEvent.MOUSE_EVENT_MASK );
    }

    public void removeActionListener( ActionListener l )
    {
        actionListener = AWTEventMulticaster.remove( actionListener, l );
    }

    public Dimension getMinimumSize()
    {
        return new Dimension( 16, 16 );
    }

    public Dimension getPreferredSize()
    {
        return new Dimension( getSize() );
    }

    public void setSize( Dimension d )
    {
        d.width = Math.max( d.width, d.height );
        d.height = d.width;
        super.setSize( d );
    }

    public void doRoll()
    {
        setValue( 1 + Math.abs( randomGenerator.nextInt() ) % 6 );
    }

    /* Value */
    public int getValue() { return fValue; }
    public void setValue( int aValue )
    {
        if( fValue != aValue && aValue >= 1 && aValue <= 6 ) {
            fValue = aValue;
            repaint();
        }
    }

    /* Tag */
    public int getTag() { return fTag; }
    public void setTag( int aTag ) { fTag = aTag; }
}
